"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var data_class_copy_1 = require("data-class-copy");
var endPoint_1 = require("./endPoint");
var route_1 = require("./route");
var escapeRegExp = require("lodash/escapeRegExp");
function regexpPath(rgx) {
    var source = rgx.source;
    var trailing = escapeRegExp("\\/?(?=\\/|$)");
    var leading = escapeRegExp("^\\");
    var escaped = escapeRegExp("\\");
    var toReplace = [
        "^" + escapeRegExp("^\\"),
        escapeRegExp("\\/?(?=\\/|$)") + "$",
        escapeRegExp("\\"),
        "^" + escapeRegExp("/?(?=/|$)")
    ];
    return toReplace.reduce(function (source, pattern) { return source.replace(new RegExp(pattern), ""); }, source);
}
var Router = (function () {
    function Router(params) {
        Object.assign(this, params);
    }
    Router_1 = Router;
    Router.parseExpressRouter = function (expressRouter) {
        var handleRoute = expressRouter.route || expressRouter.handle;
        return Router_1.empty.copy({
            routes: handleRoute != null && handleRoute.stack != null ? handleRoute.stack.map(route_1.parseExpressRoute) : [],
            path: expressRouter.path || handleRoute.path || regexpPath(expressRouter.regexp),
            expressRouter: expressRouter
        });
    };
    Router.parseExpressApp = function (app) {
        return Router_1.empty.copy({
            routes: app._router.stack.map(route_1.parseExpressRoute),
        });
    };
    Object.defineProperty(Router.prototype, "children", {
        get: function () {
            return this.routes;
        },
        enumerable: true,
        configurable: true
    });
    Router.prototype.setParentOnChildren = function () {
        var _this = this;
        this.routes = this.routes.map(function (route) {
            route.router = _this;
            if (Router_1.isInstanceOf(route)) {
                route.setParentOnChildren();
            }
            return route;
        });
        return this;
    };
    Object.defineProperty(Router.prototype, "fullPath", {
        get: function () {
            var path = "";
            var router = this.router;
            while (router != null) {
                path = router.path + path;
                router = router.router;
            }
            return path;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Router.prototype, "toEndPoints", {
        get: function () {
            return endPoint_1.EndPoint.flatRoute(this);
        },
        enumerable: true,
        configurable: true
    });
    var Router_1;
    Router.empty = new Router_1({
        path: "",
        routes: [],
        expressRouter: null
    });
    Router = Router_1 = tslib_1.__decorate([
        data_class_copy_1.dataClass
    ], Router);
    return Router;
}());
exports.Router = Router;
//# sourceMappingURL=router.js.map